@@ -21,6 +21,7 @@ const QString API_OAUTH_TOKEN = QStringLiteral("/api/v1/oauth-clients/local");
 const QString API_LOGIN_TOKEN = QStringLiteral("/api/v1/users/token");
 const QString API_SUBSCRIPTION_VIDEOS = QStringLiteral("/api/v1/users/me/subscriptions/videos");
 const QString API_SUBSCRIPTIONS = QStringLiteral("/api/v1/users/me/subscriptions");
+const QString API_HISTORY = QStringLiteral("/api/v1/users/me/history/videos");
 
 const QString CLIENT_ID_KEY = QStringLiteral("client-id");
 const QString CLIENT_SECRET_KEY = QStringLiteral("client-secret");
@@ -98,6 +99,7 @@ bool PeerTubeApi::supportsFeature(AbstractApi::SupportedFeature feature)
     switch (feature) {
     case PopularPage:
     case TrendingCategories:
+    case SponsorBlock:
         return false;
     }
 
@@ -136,7 +138,7 @@ QFuture<LogInResult> PeerTubeApi::logIn(const QString &username, const QString &
     request.setHeader(QNetworkRequest::ContentTypeHeader, QByteArrayLiteral("application/x-www-form-urlencoded"));
     request.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::RedirectPolicy::ManualRedirectPolicy);
 
-    return post<LogInResult>(std::move(request), params.toString().toUtf8(), [=](QNetworkReply *reply) -> LogInResult {
+    return post<LogInResult>(std::move(request), params.toString().toUtf8(), [this](QNetworkReply *reply) -> LogInResult {
         if (reply->error() == QNetworkReply::NoError) {
             const QJsonDocument doc = QJsonDocument::fromJson(reply->readAll());
             auto obj = doc.object();
@@ -155,7 +157,7 @@ QFuture<LogInResult> PeerTubeApi::logIn(const QString &username, const QString &
 QFuture<VideoResult> PeerTubeApi::requestVideo(const QString &videoId)
 {
     // This needs to be authenticated depending on where the video id is from (like subscriptions)
-    return get<VideoResult>(authenticatedNetworkRequest(videoUrl(videoId)), [=](QNetworkReply *reply) -> VideoResult {
+    return get<VideoResult>(authenticatedNetworkRequest(videoUrl(videoId)), [this](QNetworkReply *reply) -> VideoResult {
         if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
             auto video = Video::fromJson(doc);
             fixupVideo(video);
@@ -165,11 +167,6 @@ QFuture<VideoResult> PeerTubeApi::requestVideo(const QString &videoId)
     });
 }
 
-QString PeerTubeApi::resolveVideoUrl(const QString &videoId)
-{
-    return QStringLiteral("https://%1/videos/watch/%2").arg(m_apiHost, videoId);
-}
-
 QFuture<SearchListResult> PeerTubeApi::requestSearchResults(const SearchParameters &parameters, Paginator *paginator)
 {
     QHash<QString, QString> searchParameters = {{u"search"_s, parameters.query()}};
@@ -186,7 +183,7 @@ QFuture<SearchListResult> PeerTubeApi::requestSearchResults(const SearchParamete
 
     auto request = QNetworkRequest(url);
 
-    return get<SearchListResult>(std::move(request), [=](QNetworkReply *reply) -> SearchListResult {
+    return get<SearchListResult>(std::move(request), [this, paginator](QNetworkReply *reply) -> SearchListResult {
         if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
             const auto obj = doc.object();
 
@@ -279,20 +276,49 @@ QFuture<Result> PeerTubeApi::unsubscribeFromChannel(const QString &channel)
 
 QFuture<HistoryResult> PeerTubeApi::requestHistory(Paginator *paginator)
 {
-    Q_UNUSED(paginator)
-    return {};
+    QUrlQuery query;
+
+    if (paginator != nullptr) {
+        paginator->setType(Paginator::Type::StartIndex);
+        query.addQueryItem(u"start"_s, QString::number(paginator->m_startIndex));
+    }
+
+    auto url = apiUrl(API_HISTORY);
+    url.setQuery(query);
+
+    return get<SubscriptionsResult>(authenticatedNetworkRequest(url), [this, paginator](QNetworkReply *reply) -> HistoryResult {
+        if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
+            auto array = doc.object().value("data"_L1).toArray();
+
+            if (paginator != nullptr && doc.object().contains("total"_L1)) {
+                paginator->m_total = doc.object()["total"_L1].toInt();
+            }
+
+            if (paginator != nullptr && doc.object().contains("data"_L1) && doc.object()["data"_L1].isArray()) {
+                paginator->m_count = doc.object()["data"_L1].toArray().size();
+            }
+
+            QList<QString> history;
+            std::transform(array.cbegin(), array.cend(), std::back_inserter(history), [](const QJsonValue &val) {
+                return val.toObject().value(QStringLiteral("uuid")).toString();
+            });
+            return history;
+        }
+        return invalidJsonError();
+    });
 }
 
 QFuture<Result> PeerTubeApi::markWatched(const QString &videoId)
 {
     Q_UNUSED(videoId)
+    // NOTE: can't be implemented on PeerTube
     return {};
 }
 
 QFuture<Result> PeerTubeApi::markUnwatched(const QString &videoId)
 {
-    Q_UNUSED(videoId)
-    return {};
+    // TODO: needs id, NOT uuid. meh
+    return deleteResource<Result>(authenticatedNetworkRequest(apiUrl(API_HISTORY % u'/' % videoId)), checkIsReplyOk);
 }
 
 QFuture<CommentsResult> PeerTubeApi::requestComments(const QString &videoId, Paginator *paginator)
@@ -301,7 +327,7 @@ QFuture<CommentsResult> PeerTubeApi::requestComments(const QString &videoId, Pag
 
     QUrl url = apiUrl(API_VIDEOS % u'/' % videoId % u"/comment-threads");
 
-    return get<CommentsResult>(authenticatedNetworkRequest(std::move(url)), [=](QNetworkReply *reply) -> CommentsResult {
+    return get<CommentsResult>(authenticatedNetworkRequest(std::move(url)), [this](QNetworkReply *reply) -> CommentsResult {
         if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
             const auto array = doc[u"data"].toArray();
 
@@ -344,7 +370,7 @@ QFuture<ChannelResult> PeerTubeApi::requestChannelInfo(const QString &queryd)
 {
     QUrl url = apiUrl(API_CHANNEL % u'/' % queryd);
 
-    return get<ChannelResult>(authenticatedNetworkRequest(std::move(url)), [=](QNetworkReply *reply) -> ChannelResult {
+    return get<ChannelResult>(authenticatedNetworkRequest(std::move(url)), [this](QNetworkReply *reply) -> ChannelResult {
         if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
             QInvidious::Channel channel;
             Channel::fromJson(doc.object(), channel);
@@ -358,6 +384,7 @@ QFuture<ChannelResult> PeerTubeApi::requestChannelInfo(const QString &queryd)
 QFuture<PlaylistsResult> PeerTubeApi::requestChannelPlaylists(const QString &channelId)
 {
     // TODO: peertube stub
+    Q_UNUSED(channelId)
     return {};
 }
 
@@ -406,7 +433,7 @@ PeerTubeApi::requestVideoList(VideoListType queryType, const QString &urlExtensi
     // PeerTube requests should always be authenticated
     auto request = authenticatedNetworkRequest(std::move(url));
 
-    return get<VideoListResult>(std::move(request), [=](QNetworkReply *reply) -> VideoListResult {
+    return get<VideoListResult>(std::move(request), [this, paginator, queryType](QNetworkReply *reply) -> VideoListResult {
         if (auto doc = QJsonDocument::fromJson(reply->readAll()); !doc.isNull()) {
             auto obj = doc.object();
             if (paginator != nullptr && obj.contains("total"_L1)) {
@@ -441,7 +468,7 @@ PeerTubeApi::requestVideoList(VideoListType queryType, const QString &urlExtensi
     });
 }
 
-QNetworkRequest PeerTubeApi::authenticatedNetworkRequest(QUrl &&url)
+QNetworkRequest PeerTubeApi::authenticatedNetworkRequest(QUrl url)
 {
     QNetworkRequest request(url);
     if (isLoggedIn()) {
