@@ -20,6 +20,7 @@ VideoController::VideoController(QObject *parent)
     , m_videoModel(new VideoModel(this))
     , m_videoQueue(new VideoQueue(this))
     , m_mpris(new Mpris2(this))
+    , m_sponsorBlockController(new SponsorBlockController(this))
 {
     connect(m_videoModel, &VideoModel::audioUrlChanged, this, [this] {
         m_currentPlayer->setAudioUrl(m_videoModel->audioUrl());
@@ -38,10 +39,15 @@ VideoController::VideoController(QObject *parent)
         }
 
         m_videoModel->fetch(m_videoQueue->getCurrentVideoId());
+        m_sponsorBlockController->requestSponsors(m_videoQueue->getCurrentVideoId());
 
         openPlayer();
     });
     connect(this, &VideoController::playbackStateChanged, this, [this] {
+        if (!m_currentPlayer) {
+            return;
+        }
+
         const bool paused = m_currentPlayer->paused();
         const bool durationMatchesPosition = abs(m_currentPlayer->duration() - m_currentPlayer->position()) < 0.1;
         const bool notAtBeginning = m_currentPlayer->position() > 1;
@@ -78,6 +84,23 @@ VideoController::VideoController(QObject *parent)
         }
 #endif
     });
+    connect(this, &VideoController::positionChanged, this, [this] {
+        auto currentPosition = position();
+        if (m_sponsorBlockController->isSponsor(currentPosition)) {
+            auto freePosition = m_sponsorBlockController->findFreePosition(currentPosition);
+            if (freePosition.has_value()) {
+                if (PlasmaTube::instance().settings()->skipSponsorBlock()) {
+                    {
+                        QSignalBlocker blocker(this); // ensure we don't enter a super loop
+                        m_currentPlayer->setPosition(freePosition.value());
+                    }
+                    Q_EMIT skippedSponsorSegment();
+                } else {
+                    Q_EMIT enteredSponsorSegment(freePosition.value());
+                }
+            }
+        }
+    });
 }
 
 void VideoController::play(const QString &videoId)
