@@ -96,7 +96,7 @@ void PeerTubeInstancesModel::fill()
         if (!instances.isEmpty()) {
             QList<PeerTubeInstance> fetchedInstances;
 
-            std::transform(instances.cbegin(), instances.cend(), std::back_inserter(fetchedInstances), [=](const QVariant &value) -> auto {
+            std::transform(instances.cbegin(), instances.cend(), std::back_inserter(fetchedInstances), [this](const QVariant &value) -> auto {
                 return fromSourceData(value.toJsonObject());
             });
             beginInsertRows({}, m_instances.size(), m_instances.size() + fetchedInstances.size() - 1);
